/**************************************************************************
*  Copyright (c) 2018 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  24.02.2018  mifi  First Version.
**************************************************************************/
#define __FSL_QSPI_NOR_BOOT_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdint.h>


/*=======================================================================*/
/*  Definition of all extern Data                                        */
/*=======================================================================*/

extern void reset_handler(void);


/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*
 * Image Vector Table Structure
 */
typedef struct _ivt_
{
   uint32_t header;
   uint32_t entry;
   uint32_t reserved1;
   uint32_t dcd;
   uint32_t boot_data;
   uint32_t self;
   uint32_t csf;
   uint32_t reserved2;
} ivt_t;


/*
 * Boot Data Structure
 */
typedef struct _boot_data_
{
  uint32_t start;
  uint32_t length;
  uint32_t plugin;
} boot_data_t;


/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

#if defined(XIP_BOOT_HEADER_ENABLE) && (XIP_BOOT_HEADER_ENABLE == 1)

/*
 * Device Configuration Data (DCD)
 */
__attribute__((section(".boot_hdr.dcd_data")))
const uint8_t dcd_data[496] = 
{
   0xd2, 0x01, 0xf0, 0x40, 
   0xcc, 0x01, 0xec, 0x04, 
   
   0x02, 0x0c, 0x40, 0x68, 0xff, 0xff, 0xff, 0xff,    // CCM_CGPR0 
   0x02, 0x0c, 0x40, 0x6c, 0xff, 0xff, 0xff, 0xff,    // CCM_CGPR1
   0x02, 0x0c, 0x40, 0x70, 0xff, 0xff, 0xff, 0xff,    // CCM_CGPR2
   0x02, 0x0c, 0x40, 0x74, 0xff, 0xff, 0xff, 0xff,    // CCM_CGPR3
   0x02, 0x0c, 0x40, 0x78, 0xff, 0xff, 0xff, 0xff,    // CCM_CGPR4
   0x02, 0x0c, 0x40, 0x7c, 0xff, 0xff, 0xff, 0xff,    // CCM_CGPR5
   0x02, 0x0c, 0x40, 0x80, 0xff, 0xff, 0xff, 0xff,    // CCM_CGPR6
   0x02, 0x0c, 0x40, 0x84, 0xff, 0xff, 0xff, 0xff,    // ???
   
   0x02, 0x0e, 0x04, 0xb4, 0x00, 0x0c, 0x00, 0x00,    // IOMUXC_SW_PAD_CTL_GRP_DDR_TYPE
   0x02, 0x0e, 0x04, 0xac, 0x00, 0x00, 0x00, 0x00,    // IOMUXC_SW_PAD_CTL_GRP_DDRPKE  
   0x02, 0x0e, 0x02, 0x7c, 0x00, 0x00, 0x00, 0x08,    // IOMUXC_SW_PAD_CTL_PAD_DRAM_SDCLK0_P
   0x02, 0x0e, 0x02, 0x50, 0x00, 0x00, 0x00, 0x30,    // IOMUXC_SW_PAD_CTL_PAD_DRAM_CAS_B
   0x02, 0x0e, 0x02, 0x4c, 0x00, 0x00, 0x00, 0x30,    // IOMUXC_SW_PAD_CTL_PAD_DRAM_RAS_B
   0x02, 0x0e, 0x04, 0x90, 0x00, 0x00, 0x00, 0x30,    // IOMUXC_SW_PAD_CTL_GRP_ADDDS
   0x02, 0x0e, 0x02, 0x88, 0x00, 0x00, 0x00, 0x30,    // IOMUXC_SW_PAD_CTL_PAD_DRAM_RESET
   0x02, 0x0e, 0x02, 0x70, 0x00, 0x00, 0x00, 0x00,    // IOMUXC_SW_PAD_CTL_PAD_DRAM_SDBA2
   0x02, 0x0e, 0x02, 0x60, 0x00, 0x00, 0x00, 0x30,    // IOMUXC_SW_PAD_CTL_PAD_DRAM_ODT0
   0x02, 0x0e, 0x02, 0x64, 0x00, 0x00, 0x00, 0x30,    // IOMUXC_SW_PAD_CTL_PAD_DRAM_ODT1
   0x02, 0x0e, 0x04, 0xa0, 0x00, 0x00, 0x00, 0x30,    // IOMUXC_SW_PAD_CTL_GRP_CTLDS
   0x02, 0x0e, 0x04, 0x94, 0x00, 0x02, 0x00, 0x00,    // IOMUXC_SW_PAD_CTL_GRP_DDRMODE_CTL
   0x02, 0x0e, 0x02, 0x80, 0x00, 0x00, 0x00, 0x38,    // IOMUXC_SW_PAD_CTL_PAD_DRAM_SDQS0_P
   0x02, 0x0e, 0x02, 0x84, 0x00, 0x00, 0x00, 0x30,    // IOMUXC_SW_PAD_CTL_PAD_DRAM_SDQS1_P
   0x02, 0x0e, 0x04, 0xb0, 0x00, 0x02, 0x00, 0x00,    // IOMUXC_SW_PAD_CTL_GRP_DDRMODE
   0x02, 0x0e, 0x04, 0x98, 0x00, 0x00, 0x00, 0x30,    // IOMUXC_SW_PAD_CTL_GRP_B0DS
   0x02, 0x0e, 0x04, 0xa4, 0x00, 0x00, 0x00, 0x30,    // IOMUXC_SW_PAD_CTL_GRP_B1DS
   0x02, 0x0e, 0x02, 0x44, 0x00, 0x00, 0x00, 0x30,    // IOMUXC_SW_PAD_CTL_PAD_DRAM_DQM0
   0x02, 0x0e, 0x02, 0x48, 0x00, 0x00, 0x00, 0x30,    // IOMUXC_SW_PAD_CTL_PAD_DRAM_DQM1
   
   0x02, 0x1b, 0x00, 0x1c, 0x00, 0x00, 0x80, 0x00,    // MMDC_MDSCR
   0x02, 0x1b, 0x08, 0x00, 0xa1, 0x39, 0x00, 0x03,    // MMDC_MPZQHWCTRL
   0x02, 0x1b, 0x08, 0x0c, 0x00, 0x07, 0x00, 0x07,    // MMDC_MPWLDECTRL0
   0x02, 0x1b, 0x08, 0x3c, 0x41, 0x49, 0x01, 0x45,    // MMDC_MPDGCTRL0
   0x02, 0x1b, 0x08, 0x48, 0x40, 0x40, 0x45, 0x46,    // MMDC_MPRDDLCTL
   0x02, 0x1b, 0x08, 0x50, 0x40, 0x40, 0x52, 0x4d,    // MMDC_MPWRDLCTL
   0x02, 0x1b, 0x08, 0x1c, 0x33, 0x33, 0x33, 0x33,    // MMDC_MPRDDQBY0DL
   0x02, 0x1b, 0x08, 0x20, 0x33, 0x33, 0x33, 0x33,    // MMDC_MPRDDQBY1DL
   0x02, 0x1b, 0x08, 0x2c, 0xf3, 0x33, 0x33, 0x33,    // MMDC_MPWRDQBY0DL
   0x02, 0x1b, 0x08, 0x30, 0xf3, 0x33, 0x33, 0x33,    // MMDC_MPWRDQBY1DL
   0x02, 0x1b, 0x08, 0xc0, 0x00, 0x92, 0x20, 0x12,    // MMDC_MPDCCR
   0x02, 0x1b, 0x08, 0x58, 0x00, 0x00, 0x0f, 0x00,    // MMDC_MPSDCTRL
   0x02, 0x1b, 0x08, 0xb8, 0x00, 0x00, 0x08, 0x00,    // MMDC_MPMUR0
   
   0x02, 0x1b, 0x00, 0x04, 0x00, 0x02, 0x00, 0x2d,    // MMDC_MDPDC
   0x02, 0x1b, 0x00, 0x08, 0x1b, 0x33, 0x30, 0x00,    // MMDC_MDOTC
   0x02, 0x1b, 0x00, 0x0c, 0x67, 0x6b, 0x54, 0xf3,    // MMDC_MDCFG0 
   0x02, 0x1b, 0x00, 0x10, 0xb6, 0x8e, 0x0a, 0x83,    // MMDC_MDCFG1
   0x02, 0x1b, 0x00, 0x14, 0x01, 0xff, 0x00, 0xdb,    // MMDC_MDCFG2
   0x02, 0x1b, 0x00, 0x18, 0x00, 0x21, 0x17, 0x40,    // MMDC_MDMISC
   0x02, 0x1b, 0x00, 0x1c, 0x00, 0x00, 0x80, 0x00,    // MMDC_MDSCR
   0x02, 0x1b, 0x00, 0x2c, 0x00, 0x00, 0x26, 0xd2,    // MMDC_MDRWD
   0x02, 0x1b, 0x00, 0x30, 0x00, 0x6b, 0x10, 0x23,    // MMDC_MDOR
   0x02, 0x1b, 0x00, 0x40, 0x00, 0x00, 0x00, 0x4f,    // MMDC_MDASP
   0x02, 0x1b, 0x00, 0x00, 0x84, 0x18, 0x00, 0x00,    // MMDC_MDCTL
   0x02, 0x1b, 0x00, 0x1c, 0x02, 0x00, 0x80, 0x32,    // MMDC_MDSCR
   0x02, 0x1b, 0x00, 0x1c, 0x00, 0x00, 0x80, 0x33,    // MMDC_MDSCR
   0x02, 0x1b, 0x00, 0x1c, 0x00, 0x04, 0x80, 0x31,    // MMDC_MDSCR
   0x02, 0x1b, 0x00, 0x1c, 0x15, 0x20, 0x80, 0x30,    // MMDC_MDSCR
   0x02, 0x1b, 0x00, 0x1c, 0x04, 0x00, 0x80, 0x40,    // MMDC_MDSCR
   0x02, 0x1b, 0x00, 0x20, 0x00, 0x00, 0x08, 0x00,    // MMDC_MDREF
   0x02, 0x1b, 0x08, 0x18, 0x00, 0x00, 0x02, 0x27,    // MMDC_MPODTCTRL
   0x02, 0x1b, 0x00, 0x04, 0x00, 0x02, 0x55, 0x2d,    // MMDC_MDPDC
   0x02, 0x1b, 0x04, 0x04, 0x00, 0x01, 0x10, 0x06,    // MMDC_MAPSR
   0x02, 0x1b, 0x00, 0x1c, 0x00, 0x00, 0x00, 0x00     // MMDC_MDSCR
};


/*
 * Boot Data
 */
__attribute__((section(".boot_hdr.boot_data")))
const boot_data_t boot_data = 
{
   0x60000000,             /* Absolute address of the image */
   0x02000000,             /* Size of the program image     */
   0                       /* Plugin flag                   */
};


/*
 * Image Vector Table
 */
__attribute__((section(".boot_hdr.ivt")))
const ivt_t ivt_data = 
{
   0x402000D1,                /* Header */
   (uint32_t)reset_handler,   /* Absolute address of the first instruction to execute from the image */
   0,                         /* Reserved1 */
   (uint32_t)&dcd_data,       /* Absolute address of the image DCD */
   (uint32_t)&boot_data,      /* Absolute address of the Boot Data */
   (uint32_t)&ivt_data,       /* Absolute address of the IVT */
   0,                         /* Absolute address of the Command Sequence File (CSF) */
   0                          /* Reserved2 */
};

#endif /* #if defined(XIP_BOOT_HEADER_ENABLE) && (XIP_BOOT_HEADER_ENABLE == 1) */

/*** EOF ***/
